@@ -18,13 +18,18 @@
 
 #include <epan/packet.h>
 #include <epan/conversation.h>
+#include <epan/prefs.h>
+#include <epan/expert.h>
+#include "proto_data.h"
 
 #ifdef HAVE_ZLIB
 #include <zlib.h>
 #endif
 
+void proto_reg_handoff_blip(void);
+void proto_register_blip(void);
+
 #define BLIP_BODY_CHECKSUM_SIZE 4
-#define BLIP_INFLATE_BUFFER_SIZE 16384
 
 // blip_conversation_entry_t is metadata that the blip dissector associates w/ each wireshark conversation
 typedef struct {
@@ -32,14 +37,34 @@ typedef struct {
 	// Keep track of the largest frame number seen.  This is useful for determining whether
 	// this is the first frame in a request message or not.
 
-	// key: msgtype:srcport:messagenumber -> value: frame number for the _first_ frame in this request message
-	// Example: "MSG:23243:56" -> 12
-	// which means: "the first frame for blip message number 56, originating from source port 23243, and for message type = MSG
-	//				 ... occurred in wireshark packet #12"
+	// key: msgtype:srcport:destport:messagenumber -> value: frame number for the _first_ frame in this request message
+	// Example: "MSG:23243:4984:56" -> 12
+	// which means: "the first frame for blip message number 56, originating from source port 23243,
+	//              ... and going to port 4984 for message type = MSG occurred in wireshark packet #12"
 	wmem_map_t *blip_requests;
 
+#ifdef HAVE_ZLIB
+	// The streams used to decode a particular connection.	These are per direction and per connection.
+	wmem_map_t *decompress_streams;
+#endif
 } blip_conversation_entry_t;
 
+#ifdef HAVE_ZLIB
+typedef enum
+{
+	no_error = 0,
+	zlib_error,
+	overflow_error
+} decompress_error_t;
+
+typedef struct
+{
+	decompress_error_t domain;
+	int code;
+	size_t size;
+	void* buf;
+} decompress_result_t;
+#endif
 
 // File level variables
 static dissector_handle_t blip_handle;
@@ -51,8 +76,10 @@ static int hf_blip_properties = -1;
 static int hf_blip_message_body = -1;
 static int hf_blip_ack_size = -1;
 static int hf_blip_checksum = -1;
+
 static gint ett_blip = -1;
-static wmem_map_t* decompress_streams = NULL;
+
+static expert_field ei_blip_decompress_buffer_error = EI_INIT;
 
 // Compressed = 0x08
 // Urgent	  = 0x10
@@ -91,6 +118,11 @@ static const val64_string msg_types[] = {
 	{ 0, NULL }
 };
 
+// Preferences
+#ifdef HAVE_ZLIB
+static guint max_uncompressed_size = 64; // Max uncompressed body size in Kb
+#endif
+
 // MSG =	0x00
 // RPY =	0x01
 // ERR =	0x02
@@ -106,7 +138,7 @@ get_message_type(guint64 value_frame_flags)
 
 static gboolean
 is_ack_message(guint64 value_frame_flags) {
-	// Note, even though this is a 64-bit int, only the least significant byte has meaningful		 information,
+	// Note, even though this is a 64-bit int, only the least significant byte has meaningful information,
 	// since frame flags all fit into one byte at the time this code was written.
 
 	// Mask out the least significant bits: 0000 0111
@@ -201,7 +233,32 @@ handle_ack_message(tvbuff_t *tvb, _U_ packet_info *pinfo, _U_ proto_tree *blip_t
 	return tvb_captured_length(tvb);
 }
 
+static blip_conversation_entry_t*
+get_blip_conversation(packet_info* pinfo)
+{
+	// Create a new conversation if needed and associate the blip_conversation_entry_t with it
+	// Adapted from sample code in doc/README.dissector
+	conversation_t *conversation;
+	conversation = find_or_create_conversation(pinfo);
+	blip_conversation_entry_t *conversation_entry_ptr = (blip_conversation_entry_t*)conversation_get_proto_data(conversation, proto_blip);
+	if (conversation_entry_ptr == NULL) {
+
+		// create a new blip_conversation_entry_t
+		conversation_entry_ptr = wmem_new(wmem_file_scope(), blip_conversation_entry_t);
+
+		// create a new hash map and save a reference in blip_conversation_entry_t
+		conversation_entry_ptr->blip_requests = wmem_map_new(wmem_file_scope(), g_str_hash, g_str_equal);
 #ifdef HAVE_ZLIB
+		conversation_entry_ptr->decompress_streams = wmem_map_new(wmem_file_scope(), g_direct_hash, g_direct_equal);
+#endif
+		conversation_add_proto_data(conversation, proto_blip, conversation_entry_ptr);
+	}
+
+	return conversation_entry_ptr;
+}
+
+#ifdef HAVE_ZLIB
+
 static gboolean
 z_stream_destroy_cb(wmem_allocator_t *allocator _U_, wmem_cb_event_t event _U_, void *user_data)
 {
@@ -210,25 +267,54 @@ z_stream_destroy_cb(wmem_allocator_t *allocator _U_, wmem_cb_event_t event _U_,
 	return FALSE;
 }
 
-static z_stream* get_decompress_stream(packet_info* pinfo)
+static z_stream*
+get_decompress_stream(packet_info* pinfo)
 {
+	const blip_conversation_entry_t* blip_convo = get_blip_conversation(pinfo);
+
 	// Store compression state per srcport/destport.
-	// XXX why not store this in the conversation?
 	guint32 hash_key = (pinfo->srcport << 16) | pinfo->destport;
-	z_stream* decompress_stream = (z_stream*)wmem_map_lookup(decompress_streams, GUINT_TO_POINTER(hash_key));
+	z_stream* decompress_stream = (z_stream *)wmem_map_lookup(blip_convo->decompress_streams, GUINT_TO_POINTER(hash_key));
 	if(decompress_stream) {
 		return decompress_stream;
 	}
 
 	decompress_stream = wmem_new0(wmem_file_scope(), z_stream);
+	wmem_map_insert(blip_convo->decompress_streams, GUINT_TO_POINTER(hash_key), decompress_stream);
 	wmem_register_callback(wmem_file_scope(), z_stream_destroy_cb, decompress_stream);
-	wmem_map_insert(decompress_streams, GUINT_TO_POINTER(hash_key), decompress_stream);
+
 	return decompress_stream;
 }
 
 static tvbuff_t*
-decompress(packet_info* pinfo, tvbuff_t* tvb, gint offset, gint length)
+decompress(packet_info* pinfo, proto_tree* tree, tvbuff_t* tvb, gint offset, gint length)
 {
+	if(PINFO_FD_VISITED(pinfo)) {
+		const decompress_result_t* saved_data = (decompress_result_t *)p_get_proto_data(wmem_file_scope(), pinfo, proto_blip, 0);
+		if(!saved_data) {
+			proto_tree_add_string(tree, hf_blip_message_body, tvb, offset, tvb_reported_length_remaining(tvb, offset), "<Error decompressing data>");
+			return NULL;
+		}
+
+		if(saved_data->domain) {
+			proto_item* field = proto_tree_add_string(tree, hf_blip_message_body, tvb, offset, tvb_reported_length_remaining(tvb, offset), "<Error decompressing data>");
+			if(saved_data->domain == zlib_error) {
+				expert_add_info_format(pinfo, field, &ei_blip_decompress_buffer_error, "Unable to decompress message, got zlib error %d", saved_data->code);
+			} else {
+				expert_add_info_format(pinfo, field, &ei_blip_decompress_buffer_error, "Unable to decompress message, buffer too small (%u Kb).  Please adjust in settings.", max_uncompressed_size);
+			}
+
+			return NULL;
+		} else {
+			tvbuff_t* decompressedChild = tvb_new_child_real_data(tvb, (guint8 *)saved_data->buf,
+				(gint)saved_data->size, (gint)saved_data->size);
+			add_new_data_source(pinfo, decompressedChild, "Decompressed Payload");
+			return decompressedChild;
+		}
+	}
+
+	static gboolean size_overflow = FALSE;
+	const guint8* buf = tvb_get_ptr(tvb, offset, length);
 	z_stream* decompress_stream = get_decompress_stream(pinfo);
 	static Byte trailer[4] = { 0x00, 0x00, 0xff, 0xff };
 	if(!decompress_stream->next_out) {
@@ -243,29 +329,64 @@ decompress(packet_info* pinfo, tvbuff_t* tvb, gint offset, gint length)
 		}
 	}
 
-	const guint8* buf = tvb_get_ptr(tvb, offset, length);
-	Bytef* decompress_buffer = (Bytef*)wmem_alloc(wmem_packet_scope(), BLIP_INFLATE_BUFFER_SIZE);
+	// Create a temporary buffer of the maximum size, which will get cleaned up later
+	// when the packet scope is freed
+	uInt buffer_size = max_uncompressed_size * 1024;
+	Bytef* decompress_buffer = (Bytef*)wmem_alloc(wmem_packet_scope(), buffer_size);
 	decompress_stream->next_in = (Bytef*)buf;
 	decompress_stream->avail_in = length;
 	decompress_stream->next_out = decompress_buffer;
-	decompress_stream->avail_out = BLIP_INFLATE_BUFFER_SIZE;
+	decompress_stream->avail_out = buffer_size;
 	uLong start = decompress_stream->total_out;
 	int err = inflate(decompress_stream, Z_NO_FLUSH);
-	if(err < 0) {
+	if(err != Z_OK) {
+		proto_item* field = proto_tree_add_string(tree, hf_blip_message_body, tvb, offset, tvb_reported_length_remaining(tvb, offset), "<Error decompressing data>");
+		decompress_result_t* data_to_save = wmem_new0(wmem_file_scope(), decompress_result_t);
+		if(size_overflow && err == Z_DATA_ERROR) {
+			data_to_save->domain = overflow_error;
+			expert_add_info_format(pinfo, field, &ei_blip_decompress_buffer_error, "Unable to decompress message, buffer too small (%u Kb).  Please adjust in settings.", max_uncompressed_size);
+		} else {
+			data_to_save->domain = zlib_error;
+			data_to_save->code = err;
+			expert_add_info_format(pinfo, field, &ei_blip_decompress_buffer_error, "Unable to decompress message, got zlib error %d", err);
+		}
+
+		p_add_proto_data(wmem_file_scope(), pinfo, proto_blip, 0, data_to_save);
 		return NULL;
 	}
 
 	decompress_stream->next_in = trailer;
 	decompress_stream->avail_in = 4;
 	err = inflate(decompress_stream, Z_SYNC_FLUSH);
-	if(err < 0) {
+	if(err != Z_OK) {
+		proto_item* field = proto_tree_add_string(tree, hf_blip_message_body, tvb, offset, tvb_reported_length_remaining(tvb, offset), "<Error decompressing data>");
+		decompress_result_t* data_to_save = wmem_new0(wmem_file_scope(), decompress_result_t);
+		if(err == Z_BUF_ERROR) {
+			data_to_save->domain = overflow_error;
+			size_overflow = TRUE;
+			expert_add_info_format(pinfo, field, &ei_blip_decompress_buffer_error, "Unable to decompress message, buffer too small (%u Kb).  Please adjust in settings.", max_uncompressed_size);
+		} else {
+			data_to_save->domain = zlib_error;
+			data_to_save->code = err;
+			expert_add_info_format(pinfo, field, &ei_blip_decompress_buffer_error, "Unable to decompress message, got zlib error %d", err);
+		}
+
+		p_add_proto_data(wmem_file_scope(), pinfo, proto_blip, 0, data_to_save);
 		return NULL;
 	}
 
+	// Shrink the buffer so that there is not wasted space on the end of it since
+	// it will be long lived in the file scope
 	uLong bodyLength = decompress_stream->total_out - start;
-	guint8* poolBuffer = (guint8*)wmem_memdup(pinfo->pool, decompress_buffer, bodyLength);
-	tvbuff_t* decompressedChild = tvb_new_child_real_data(tvb, poolBuffer, (guint)bodyLength, (gint)bodyLength);
+	Bytef* shortened_buffer = (Bytef *)wmem_memdup(wmem_file_scope(), decompress_buffer, bodyLength);
+
+	tvbuff_t* decompressedChild = tvb_new_child_real_data(tvb, shortened_buffer, (guint)bodyLength, (gint)bodyLength);
 	add_new_data_source(pinfo, decompressedChild, "Decompressed Payload");
+	decompress_result_t* data_to_save = wmem_new0(wmem_file_scope(), decompress_result_t);
+	data_to_save->size = (size_t)bodyLength;
+	data_to_save->buf = shortened_buffer;
+	p_add_proto_data(wmem_file_scope(), pinfo, proto_blip, 0, data_to_save);
+
 	return decompressedChild;
 }
 #endif /* HAVE_ZLIB */
@@ -275,7 +396,7 @@ dissect_blip(tvbuff_t *tvb, _U_ packet_info *pinfo, proto_tree *tree, _U_ void *
 {
 
 	proto_tree *blip_tree;
-	gint		offset = 0;
+	gint offset = 0;
 
 	/* Set the protcol column to say BLIP */
 	col_set_str(pinfo->cinfo, COL_PROTOCOL, "BLIP");
@@ -337,20 +458,7 @@ dissect_blip(tvbuff_t *tvb, _U_ packet_info *pinfo, proto_tree *tree, _U_ void *
 
 	// ------------------------------------- Conversation Tracking -----------------------------------------------------
 
-	// Create a new conversation if needed and associate the blip_conversation_entry_t with it
-	// Adapted from sample code in https://raw.githubusercontent.com/wireshark/wireshark/master/doc/README.dissector
-	conversation_t *conversation;
-	conversation = find_or_create_conversation(pinfo);
-	blip_conversation_entry_t *conversation_entry_ptr = (blip_conversation_entry_t*)conversation_get_proto_data(conversation, proto_blip);
-	if (conversation_entry_ptr == NULL) {
-
-		// create a new blip_conversation_entry_t
-		conversation_entry_ptr = wmem_new(wmem_file_scope(), blip_conversation_entry_t);
-
-		// create a new hash map and save a reference in blip_conversation_entry_t
-		conversation_entry_ptr->blip_requests = wmem_map_new(wmem_file_scope(), g_str_hash, g_str_equal);
-
-	}
+	blip_conversation_entry_t *conversation_entry_ptr = get_blip_conversation(pinfo);
 
 	// Is this the first frame in a blip message with multiple frames?
 	gboolean first_frame_in_msg = is_first_frame_in_msg(
@@ -360,27 +468,17 @@ dissect_blip(tvbuff_t *tvb, _U_ packet_info *pinfo, proto_tree *tree, _U_ void *
 			value_message_num
 	);
 
-	// Update the conversation w/ the latest version of the blip_conversation_entry_t
-	conversation_add_proto_data(conversation, proto_blip, (void *)conversation_entry_ptr);
-
 	tvbuff_t* tvb_to_use = tvb;
 	gboolean compressed = is_compressed(value_frame_flags);
-	if(compressed) {
-		if(!tree) {
-			// This is just a preliminary pass, no protocol tree to fill in so
-			// don't waste time decompressing (it will probably mess things up
-			// anyway)
-			return tvb_reported_length(tvb);
-		}
 
+	if(compressed) {
 #ifdef HAVE_ZLIB
-		tvb_to_use = decompress(pinfo, tvb, offset, tvb_reported_length_remaining(tvb, offset) - BLIP_BODY_CHECKSUM_SIZE);
+		tvb_to_use = decompress(pinfo, blip_tree, tvb, offset, tvb_reported_length_remaining(tvb, offset) - BLIP_BODY_CHECKSUM_SIZE);
 		if(!tvb_to_use) {
-			proto_tree_add_string(blip_tree, hf_blip_message_body, tvb, offset, tvb_reported_length_remaining(tvb, offset), "<Error decompressing message>");
 			return tvb_reported_length(tvb);
 		}
 #else /* ! HAVE_ZLIB */
-		proto_tree_add_string(blip_tree, hf_blip_message_body, tvb, offset, tvb_reported_length_remaining(tvb, offset), "<decompression support is not available>");
+		proto_tree_add_string(tree, hf_blip_message_body, tvb, offset, tvb_reported_length_remaining(tvb, offset), "<decompression support is not available>");
 		return tvb_reported_length(tvb);
 #endif /* ! HAVE_ZLIB */
 
@@ -450,7 +548,6 @@ dissect_blip(tvbuff_t *tvb, _U_ packet_info *pinfo, proto_tree *tree, _U_ void *
 		proto_tree_add_item(blip_tree, hf_blip_message_body, tvb_to_use, offset, reported_length_remaining, ENC_UTF_8|ENC_NA);
 	}
 
-	offset += reported_length_remaining;
 	proto_tree_add_item(blip_tree, hf_blip_checksum, tvb, tvb_reported_length(tvb) - BLIP_BODY_CHECKSUM_SIZE, BLIP_BODY_CHECKSUM_SIZE, ENC_BIG_ENDIAN);
 
 	// -------------------------------------------- Etc ----------------------------------------------------------------
@@ -497,15 +594,29 @@ proto_register_blip(void)
 		&ett_blip
 	};
 
+	/* Expert Infos */
+	static ei_register_info ei[] = {
+		{ &ei_blip_decompress_buffer_error, { "blip.decompress_buffer_error", PI_UNDECODED, PI_WARN, "Decompression error", EXPFILL }}
+	};
+
 	proto_blip = proto_register_protocol("BLIP Couchbase Mobile", "BLIP", "blip");
+	expert_module_t* expert_blip = expert_register_protocol(proto_blip);
 
 	proto_register_field_array(proto_blip, hf, array_length(hf));
 	proto_register_subtree_array(ett, array_length(ett));
+	expert_register_field_array(expert_blip, ei, array_length(ei));
 
 	blip_handle = register_dissector("blip", dissect_blip, proto_blip);
 
-	decompress_streams = wmem_map_new_autoreset(wmem_epan_scope(), wmem_file_scope(), g_direct_hash, g_direct_equal);
-
+#ifdef HAVE_ZLIB
+	module_t *blip_module = prefs_register_protocol(proto_blip, NULL);
+	prefs_register_uint_preference(blip_module, "max_uncompressed_size",
+						"Maximum uncompressed message size (Kb)",
+						"The maximum size of the buffer for uncompressed messages. "
+						"If a message is larger than this, then the packet containing "
+						"the message, as well as subsequent packets, will fail to "
+						"decompress", 10, &max_uncompressed_size);
+#endif
 }
 
 void
