@@ -950,12 +950,14 @@ dissect_netlink_route_ifa_attrs(tvbuff_t *tvb, struct netlink_route_info *info _
 			proto_tree_add_item(tree, &hfi_netlink_route_ifa_flags32, tvb, offset, 4, info->encoding);
 			return 1;
 		case WS_IFA_ADDRESS:
+		case WS_IFA_LOCAL:
+		case WS_IFA_BROADCAST:
 			if (len == 4) {
 				proto_item_append_text(tree, ": %s", tvb_ip_to_str(tvb, offset));
-				proto_tree_add_item(tree, &hfi_netlink_route_ifa_addr4, tvb, offset, len, info->encoding);
+				proto_tree_add_item(tree, &hfi_netlink_route_ifa_addr4, tvb, offset, len, ENC_BIG_ENDIAN);
 			} else {
 				proto_item_append_text(tree, ": %s", tvb_ip6_to_str(tvb, offset));
-				proto_tree_add_item(tree, &hfi_netlink_route_ifa_addr6, tvb, offset, len, info->encoding);
+				proto_tree_add_item(tree, &hfi_netlink_route_ifa_addr6, tvb, offset, len, ENC_NA);
 			}
 			return 1;
 		default:
