@@ -88,8 +88,8 @@ sub read_repo_info {
 
 	# Tarball produced by 'git archive' will have the $Format string
 	# substituted due to the use of 'export-subst' in .gitattributes.
-	my $git_archive_commit = '643e3b0f718685b3fa7008a0c5c4707daf63ca99';
-	my @git_refs = split(/, /, 'HEAD -> refs/heads/master-3.0, tag: wireshark-3.0.13, tag: v3.0.13');
+	my $git_archive_commit = 'f6557718c0ef33558b0153c146f165eb7a941394';
+	my @git_refs = split(/, /, 'HEAD -> master-3.0, tag: wireshark-3.0.14, tag: v3.0.14');
 	if (substr($git_archive_commit, 0, 1) eq '$') {
 		# If $Format is still present, then this is not a git archive.
 		$git_archive_commit = undef;
